`timescale 1ns/1ps
module top(
    input rstn,
    input sdio_clk,
    inout io_cmd_di,
    inout io_dat0_do,
    inout io_dat1_irq,
    inout io_dat2_rw,
    inout io_dat3_cs,
    
    input cpu_clk,
    
    
    output reg capture_o,

    output dat0_o,
    output dat1_o,
    output dat2_o,
    output dat3_o,
    
    output reg led[3:0],
    
    input SW1,
    input SW2,
    output reg user_sdio_cmd53_wr_en,
    output reg user_sdio_cmd53_fn_num,
    output reg [16:0] user_sdio_cmd53_addr,
    output reg [11:0] user_sdio_cmd53_len,
    output reg user_sdio_cmd53_op_code,
    output reg user_sdio_cmd53_wr_valid,
    output reg [7:0] user_sdio_cmd53_wr_data,
    output reg user_sdio_cmd53_wr_end,
    output reg user_sdio_cmd53_wr_ok,
    output reg sdio_buffer_full_reg,
    output reg user_sdio_cmd53_rd_en,
    output reg user_sdio_cmd53_rd_valid,
    output reg user_sdio_cmd53_rd_ready,
    output reg [7:0] user_sdio_cmd53_rd_data,
    output reg user_sdio_cmd53_rd_end,
    output reg user_sdio_cmd53_rd_abort,
    output reg user_sdio_cmd53_wr_abort,
    output reg [11:0] user_sdio_cmd53_cnt,
    output reg user_sdio_cmd52_if_user_sdio_cmd52_cs,
    output reg user_sdio_cmd52_if_user_sdio_cmd52_r_w,
    output reg user_sdio_cmd52_if_user_sdio_cmd52_fn_num,
    output reg user_sdio_cmd52_if_user_sdio_cmd52_raw,
    output reg [16:0] user_sdio_cmd52_if_user_sdio_cmd52_addr,
    output reg [7:0] user_sdio_cmd52_if_user_sdio_cmd52_wr_data,
    output reg [7:0] user_sdio_cmd52_if_user_sdio_cmd52_rd_data,
    output reg user_sdio_cmd52_if_user_sdio_cmd52_ack
);

wire clk, sym_clk;

wire cpu_rst;
wire slv_cpu_cs;
wire slv_cpu_op;
wire [7:0] slv_cpu_addr;
wire [31:0] slv_cpu_wr_data;
wire [3:0] slv_cpu_byte_en;
wire [31:0] slv_cpu_rd_data;
wire slv_cpu_ack;
wire slv_cpu_err;


//-----cmd52 if----
wire          sdio_cmd52_cs;
wire          sdio_cmd52_r_w;
wire          sdio_cmd52_fn_num;
wire          sdio_cmd52_raw;//1:The Read after Write flag
wire [16:0]   sdio_cmd52_addr;
wire [7:0]    sdio_cmd52_wr_data;
wire [7:0]    sdio_cmd52_rd_data;
wire          sdio_cmd52_ack;
//-----cmd53 if----
wire          sdio_cmd53_wr_en;
wire          sdio_cmd53_rd_en;
wire          sdio_cmd53_fn_num;
wire [16:0]   sdio_cmd53_addr;
wire [11:0]   sdio_cmd53_len;
wire          sdio_cmd53_op_code;

wire          sdio_cmd53_wr_valid;
wire [7:0]    sdio_cmd53_wr_data;
wire          sdio_cmd53_wr_end;
wire          sdio_cmd53_wr_ok;

wire          sdio_cmd53_rd_valid;
wire [7:0]    sdio_cmd53_rd_data;
wire          sdio_cmd53_rd_ready;

wire          sdio_buffer_full;
wire          sdio_cmd53_wr_abort;
wire          sdio_cmd53_rd_end;
wire          sdio_cmd53_rd_abort;

wire            sdio_tuning_start;
wire [7:0]      sdio_tuning_data;
wire            sdio_tuning_end;



wire cmd52_rst;
wire fun1_ioe;
wire fun1_interrupt;
wire fun1_ior;

wire sdio_cmd_in;
wire sdio_cmd_out;
wire sdio_cmd_oen;
wire sdio_dat0_in;
wire sdio_dat0_out;
wire sdio_dat0_oen;
wire sdio_dat1_in;
wire sdio_dat1_out;
wire sdio_dat1_oen;
wire sdio_dat2_in;
wire sdio_dat2_out;
wire sdio_dat2_oen;
wire sdio_dat3_in;
wire sdio_dat3_out;
wire sdio_dat3_oen;

assign cpu_rst = 1'b0;
assign clk = sym_clk;

SDIO_SDR104_Slave_Controller_Top u_sdio_slave_ctrl(
    .sdio_clk(sdio_clk),
    .sym_clk(sym_clk),
    .clk_2mhz(1'b0),
    .rstn(rstn),

    .sdio_cmd_in(sdio_cmd_in),
    .sdio_cmd_out(sdio_cmd_out),
    .sdio_cmd_oen(sdio_cmd_oen),
    .sdio_dat0_in(sdio_dat0_in),
    .sdio_dat0_out(sdio_dat0_out),
    .sdio_dat0_oen(sdio_dat0_oen),
    .sdio_dat1_in(sdio_dat1_in),
    .sdio_dat1_out(sdio_dat1_out),
    .sdio_dat1_oen(sdio_dat1_oen),
    .sdio_dat2_in(sdio_dat2_in),
    .sdio_dat2_out(sdio_dat2_out),
    .sdio_dat2_oen(sdio_dat2_oen),
    .sdio_dat3_in(sdio_dat3_in),
    .sdio_dat3_out(sdio_dat3_out),
    .sdio_dat3_oen(sdio_dat3_oen),

    //----fnx reset output----
    .cmd52_rst(cmd52_rst),//sync with sd_clk,one cycle high
    .fun1_ioe(fun1_ioe),//sync with sd_clk,is cccr IOEx
    .fun1_interrupt(fun1_interrupt),
    .fun1_ior(fun1_ior),

    //----cpu if---
    .cpu_rst(cpu_rst),//sync with cpu_clk
    .cpu_clk(cpu_clk),
    .slv_cpu_cs(slv_cpu_cs),
    .slv_cpu_op(slv_cpu_op),
    .slv_cpu_addr(slv_cpu_addr), 
    .slv_cpu_wr_data(slv_cpu_wr_data),
    .slv_cpu_byte_en(slv_cpu_byte_en),
    .slv_cpu_rd_data(slv_cpu_rd_data),
    .slv_cpu_ack(slv_cpu_ack),
    .slv_cpu_err(slv_cpu_err),
    
    //-----cmd52 if----
    .sdio_cmd52_cs(sdio_cmd52_cs),
    .sdio_cmd52_r_w(sdio_cmd52_r_w),
    .sdio_cmd52_fn_num(sdio_cmd52_fn_num),
    .sdio_cmd52_raw(sdio_cmd52_raw),
    .sdio_cmd52_addr(sdio_cmd52_addr),
    .sdio_cmd52_wr_data(sdio_cmd52_wr_data),
    .sdio_cmd52_rd_data(sdio_cmd52_rd_data),
    .sdio_cmd52_ack(sdio_cmd52_ack),
    //-----cmd53 if----
    .sdio_cmd53_wr_en(sdio_cmd53_wr_en),
    .sdio_cmd53_rd_en(sdio_cmd53_rd_en),
    .sdio_cmd53_fn_num(sdio_cmd53_fn_num),
    .sdio_cmd53_addr(sdio_cmd53_addr),
    .sdio_cmd53_len(sdio_cmd53_len),
    .sdio_cmd53_op_code(sdio_cmd53_op_code),

    .sdio_cmd53_wr_valid(sdio_cmd53_wr_valid),
    .sdio_cmd53_wr_data(sdio_cmd53_wr_data),
    .sdio_cmd53_wr_end(sdio_cmd53_wr_end),
    .sdio_cmd53_wr_ok(sdio_cmd53_wr_ok),
    .sdio_cmd53_wr_abort(sdio_cmd53_wr_abort),

    .sdio_cmd53_rd_valid(sdio_cmd53_rd_valid),
    .sdio_cmd53_rd_data(sdio_cmd53_rd_data),
    .sdio_cmd53_rd_ready(sdio_cmd53_rd_ready),
    .sdio_cmd53_rd_end(sdio_cmd53_rd_end),
    .sdio_cmd53_rd_abort(sdio_cmd53_rd_abort),
    
    .sdio_buffer_full(sdio_buffer_full),
    
    .sdio_tuning_start(sdio_tuning_start),
    .sdio_tuning_data(sdio_tuning_data),
    .sdio_tuning_end(sdio_tuning_end)

    

);


user_sdio_cmd53_if u_user_sdio_cmd53_if(
//--------clock and reset---------
    .clk(clk),  //sd clock
    .rstn(rstn), //system rstn
    .fun1_rst(!fun1_ioe), 
    //cmd53
    .sdio_cmd53_wr_en(sdio_cmd53_wr_en),
    .sdio_cmd53_rd_en(sdio_cmd53_rd_en),
    .sdio_cmd53_fn_num(sdio_cmd53_fn_num),
    .sdio_cmd53_addr(sdio_cmd53_addr),
    .sdio_cmd53_len(sdio_cmd53_len),
    .sdio_cmd53_op_code(sdio_cmd53_op_code),
    
    .sdio_cmd53_wr_valid(sdio_cmd53_wr_valid),
    .sdio_cmd53_wr_data(sdio_cmd53_wr_data),
    .sdio_cmd53_wr_end(sdio_cmd53_wr_end),
    .sdio_cmd53_wr_ok(sdio_cmd53_wr_ok),
    .sdio_cmd53_wr_abort(sdio_cmd53_wr_abort),
    
    .sdio_cmd53_rd_valid(sdio_cmd53_rd_valid),
    .sdio_cmd53_rd_data(sdio_cmd53_rd_data),
    .sdio_cmd53_rd_ready(sdio_cmd53_rd_ready),
    .sdio_cmd53_rd_end(sdio_cmd53_rd_end),
    .sdio_cmd53_rd_abort(sdio_cmd53_rd_abort),
    .sdio_buffer_full(sdio_buffer_full)
);

user_sdio_cmd52_if u_user_sdio_cmd52_if(
//--------clock and reset---------
    .clk(clk),  //sd clock
    .rstn(rstn), //system rstn
    .fun1_rst(!fun1_ioe),   
    
    .sdio_cmd52_cs(sdio_cmd52_cs),
    .sdio_cmd52_r_w(sdio_cmd52_r_w),
    .sdio_cmd52_fn_num(sdio_cmd52_fn_num),
    .sdio_cmd52_raw(sdio_cmd52_raw),//1:The Read after Write flag
    .sdio_cmd52_addr(sdio_cmd52_addr),
    .sdio_cmd52_wr_data(sdio_cmd52_wr_data),
    .sdio_cmd52_rd_data(sdio_cmd52_rd_data),
    .sdio_cmd52_ack(sdio_cmd52_ack)
);


user_tuning_pattern u_user_tuning_pattern(
//--------clock and reset---------
    .clk(clk),  //sd clock
    .rstn(rstn), //system rstn
    .cmd52_rst(1'b0),   //cmd52 rst

    .sdio_tuning_start(sdio_tuning_start),
    .sdio_tuning_data(sdio_tuning_data),
    .sdio_tuning_end(sdio_tuning_end)
);

wire            int_ctrl;
wire            ior_ctrl;

deUstb u_deUstb(
    .out(int_ctrl),
    .in(SW1),
    .clk(clk),
    .rstn(rstn)
);

deUstb u_deUstb1(
    .out(ior_ctrl),
    .in(SW2),
    .clk(clk),
    .rstn(rstn)
);

user_cpu_slave_reg_if u_user_cpu_slave_reg_if(
    .rstn(rstn),
    .cpu_rst(cpu_rst),
    .cpu_clk(cpu_clk),
    .slv_cpu_cs(slv_cpu_cs),
    .slv_cpu_op(slv_cpu_op),
    .slv_cpu_addr(slv_cpu_addr), 
    .slv_cpu_wr_data(slv_cpu_wr_data),
    .slv_cpu_byte_en(slv_cpu_byte_en),
    .slv_cpu_rd_data(slv_cpu_rd_data),
    .slv_cpu_ack(slv_cpu_ack),
    .slv_cpu_err(slv_cpu_err),
    .int_ctrl(int_ctrl),
    .ior_ctrl(ior_ctrl),
    .fun1_rst(!fun1_ioe),  
    .sdio_cmd52_cs(sdio_cmd52_cs),
    .sdio_cmd52_r_w(sdio_cmd52_r_w),
    .sdio_cmd52_fn_num(sdio_cmd52_fn_num),
    .sdio_cmd52_addr(sdio_cmd52_addr),
    .sdio_cmd52_wr_data(sdio_cmd52_wr_data),
    .fun1_interrupt(fun1_interrupt),
    .fun1_ior(fun1_ior)
);

//------ inout ------
assign sdio_cmd_in = io_cmd_di;
assign io_cmd_di = (!sdio_cmd_oen) ? sdio_cmd_out : 1'bz;

assign sdio_dat0_in = io_dat0_do;
assign io_dat0_do = (!sdio_dat0_oen) ? sdio_dat0_out : 1'bz;

assign sdio_dat1_in = io_dat1_irq;
assign io_dat1_irq = (!sdio_dat1_oen) ? sdio_dat1_out : 1'bz;

assign sdio_dat2_in = io_dat2_rw;
assign io_dat2_rw = (!sdio_dat2_oen) ? sdio_dat2_out : 1'bz;

assign sdio_dat3_in = io_dat3_cs;
assign io_dat3_cs = (!sdio_dat3_oen) ? sdio_dat3_out : 1'bz;

//------ logic analyzer-----
assign dat0_o = sdio_dat0_in;
assign dat1_o = sdio_dat1_in;
assign dat2_o = sdio_dat2_in;
assign dat3_o = sdio_dat3_in;

//------led------
reg [24:0] led_reg;

always@(posedge cpu_clk) begin
    led_reg <= led_reg + 1'b1;
    if (led_reg == {25{1'b1}}) begin
        led[3] <= ~led[3];
        led[2] <= ~led[2];
        led[1] <= ~led[1];
        led[0] <= ~led[0];
    end
end

wire [112:0] capture_wire = {user_sdio_cmd52_if_user_sdio_cmd52_cs,user_sdio_cmd52_if_user_sdio_cmd52_r_w,user_sdio_cmd52_if_user_sdio_cmd52_fn_num,user_sdio_cmd52_if_user_sdio_cmd52_raw,user_sdio_cmd52_if_user_sdio_cmd52_addr,user_sdio_cmd52_if_user_sdio_cmd52_wr_data,user_sdio_cmd52_if_user_sdio_cmd52_rd_data,
user_sdio_cmd52_if_user_sdio_cmd52_ack,user_sdio_cmd53_wr_en,user_sdio_cmd53_rd_en,user_sdio_cmd53_fn_num,user_sdio_cmd53_addr,user_sdio_cmd53_len,user_sdio_cmd53_op_code,user_sdio_cmd53_wr_valid,user_sdio_cmd53_wr_data,user_sdio_cmd53_wr_end,user_sdio_cmd53_wr_ok,user_sdio_cmd53_wr_abort,user_sdio_cmd53_rd_valid,user_sdio_cmd53_rd_data,user_sdio_cmd53_rd_ready,user_sdio_cmd53_rd_end,user_sdio_cmd53_rd_abort,user_sdio_cmd53_cnt};

always@(posedge clk) capture_o <= ^capture_wire;

always@(posedge clk) begin
    user_sdio_cmd52_if_user_sdio_cmd52_cs      <= u_user_sdio_cmd52_if.sdio_cmd52_cs;
    user_sdio_cmd52_if_user_sdio_cmd52_r_w     <= u_user_sdio_cmd52_if.sdio_cmd52_r_w;
    user_sdio_cmd52_if_user_sdio_cmd52_fn_num  <= u_user_sdio_cmd52_if.sdio_cmd52_fn_num;
    user_sdio_cmd52_if_user_sdio_cmd52_raw     <= u_user_sdio_cmd52_if.sdio_cmd52_raw;
    user_sdio_cmd52_if_user_sdio_cmd52_addr    <= u_user_sdio_cmd52_if.sdio_cmd52_addr;
    user_sdio_cmd52_if_user_sdio_cmd52_wr_data <= u_user_sdio_cmd52_if.sdio_cmd52_wr_data;
    user_sdio_cmd52_if_user_sdio_cmd52_rd_data <= u_user_sdio_cmd52_if.sdio_cmd52_rd_data;
    user_sdio_cmd52_if_user_sdio_cmd52_ack     <= u_user_sdio_cmd52_if.sdio_cmd52_ack;
end

always@(posedge clk) begin
    sdio_buffer_full_reg <= u_user_sdio_cmd53_if.sdio_buffer_full;
end

always@(posedge clk) begin
    user_sdio_cmd53_wr_en       <= sdio_cmd53_wr_en   ;
    user_sdio_cmd53_rd_en       <= sdio_cmd53_rd_en   ;
    user_sdio_cmd53_fn_num      <= sdio_cmd53_fn_num  ;
    user_sdio_cmd53_addr        <= sdio_cmd53_addr    ;
    user_sdio_cmd53_len         <= sdio_cmd53_len     ;
    user_sdio_cmd53_op_code     <= sdio_cmd53_op_code ;
                                   
    user_sdio_cmd53_wr_valid    <= sdio_cmd53_wr_valid;
    user_sdio_cmd53_wr_data     <= sdio_cmd53_wr_data ;
    user_sdio_cmd53_wr_end      <= sdio_cmd53_wr_end  ;
    user_sdio_cmd53_wr_ok       <= sdio_cmd53_wr_ok   ;
    user_sdio_cmd53_wr_abort    <= sdio_cmd53_wr_abort;
                                   
    user_sdio_cmd53_rd_valid    <= sdio_cmd53_rd_valid;
    user_sdio_cmd53_rd_data     <= sdio_cmd53_rd_data ;
    user_sdio_cmd53_rd_ready    <= sdio_cmd53_rd_ready;
    user_sdio_cmd53_rd_end      <= sdio_cmd53_rd_end  ;
    user_sdio_cmd53_rd_abort    <= sdio_cmd53_rd_abort;
    
    user_sdio_cmd53_cnt         <= u_user_sdio_cmd53_if.cnt;
end

endmodule