module user_cpu_slave_reg_if(
    input rstn,
    input cpu_rst,//sync with cpu_clk
    input cpu_clk,
    output reg  slv_cpu_cs,
    output reg slv_cpu_op,
    output reg [7:0] slv_cpu_addr, 
    output reg [31:0] slv_cpu_wr_data,
    output reg [3:0] slv_cpu_byte_en,
    input [31:0] slv_cpu_rd_data,
    input slv_cpu_ack,
    input slv_cpu_err,
    input int_ctrl,
    input ior_ctrl,
    //cmd52_inf
    input fun1_rst,  
    input sdio_cmd52_cs,
    input sdio_cmd52_r_w,
    input sdio_cmd52_fn_num,
    input [16:0] sdio_cmd52_addr,
    input [7:0] sdio_cmd52_wr_data,
    output reg fun1_interrupt,
    output reg fun1_ior
);

//cpu_slave_reg_addr
parameter shs_addr = 8'h00;
parameter lsc_addr = 8'h04;
parameter cccr_cis_pointer_addr = 8'h08;
parameter fbr1_cis_ptr_addr = 8'h2c;
parameter cccr_max_blk_size_addr = 8'h34;
parameter fbr1_max_blk_size_addr = 8'h34;
parameter io_ready_addr = 8'h30;

//cpu_slave_reg_data
parameter shs_data = 1'b1;
parameter lsc_data = 1'b0;
parameter cccr_cis_pointer_data = 24'h5000;
parameter fbr1_cis_ptr_data = 24'h6000;
parameter cccr_max_blk_size_data = 16'd1024;
parameter fbr1_max_blk_size_data = 16'd1024;
parameter io_ready_data = 1'b1;

reg    [9:0]    cnt;
reg             int_ctrl_reg;

wire            turn_flag;
wire            int_clr_flag;
wire            error;

wire            shs_flag;
wire            lsc_flag;
wire            cccr_cis_pointer_flag;
wire            fbr1_cis_ptr_flag;
wire            cccr_max_blk_size_flag;
wire            fbr1_max_blk_size_flag;
wire            io_ready_flag;

reg             int_clr_flag_d1,int_clr_flag_d2;

assign shs_flag = (cnt == 1 && slv_cpu_ack) ? 1'b1 : 1'b0;
assign lsc_flag = (cnt == 3 && slv_cpu_ack) ? 1'b1 : 1'b0;
assign cccr_cis_pointer_flag = (cnt == 5 && slv_cpu_ack) ? 1'b1 : 1'b0;
assign fbr1_cis_ptr_flag = (cnt == 7 && slv_cpu_ack) ? 1'b1 : 1'b0;
assign cccr_max_blk_size_flag = (cnt == 9 && slv_cpu_ack) ? 1'b1 : 1'b0;
assign fbr1_max_blk_size_flag = (cnt == 11 && slv_cpu_ack) ? 1'b1 : 1'b0;
assign io_ready_flag = (cnt == 13 && slv_cpu_ack) ? 1'b1 : 1'b0;

assign error = shs_flag ? shs_data == slv_cpu_rd_data[24] : lsc_flag ? lsc_data == slv_cpu_rd_data[22] : cccr_cis_pointer_flag ? cccr_cis_pointer_data == slv_cpu_rd_data[23:0] : fbr1_cis_ptr_flag ? fbr1_cis_ptr_data == slv_cpu_rd_data[23:0] : cccr_max_blk_size_flag ? cccr_max_blk_size_data == slv_cpu_rd_data[15:0] : fbr1_max_blk_size_flag ? fbr1_max_blk_size_data == slv_cpu_rd_data[31:16] : io_ready_flag ? io_ready_data == slv_cpu_rd_data[0] : 1'b1;

assign turn_flag = int_ctrl ^ int_ctrl_reg;
assign int_clr_flag = sdio_cmd52_cs && sdio_cmd52_r_w && sdio_cmd52_fn_num && sdio_cmd52_addr == 17'h1000 && sdio_cmd52_wr_data == 8'h01 && !fun1_rst;

always@(posedge cpu_clk or negedge rstn) begin
    if(!rstn) begin
        int_clr_flag_d1 <= 'b0;
        int_clr_flag_d2 <= 'b0;
    end
    else if(cpu_rst) begin
        int_clr_flag_d1 <= 'b0;
        int_clr_flag_d2 <= 'b0;
    end
    else begin
        int_clr_flag_d1 <= int_clr_flag;
        int_clr_flag_d2 <= int_clr_flag_d1;
    end
end        
    
always@(posedge cpu_clk or negedge rstn) begin
    if(!rstn)
        cnt <= 0;
    else if(cpu_rst)
        cnt <= 0;
    else if(cnt == 10'd1023)
        cnt <= cnt;
    else if(slv_cpu_ack)
        cnt <= cnt + 1'b1;
end

always@(posedge cpu_clk or negedge rstn) begin
    if(!rstn) begin
        int_ctrl_reg <= 1'b0;
    end
    else if(cpu_rst) begin
        int_ctrl_reg <= 1'b0;
    end
    else begin
        int_ctrl_reg <= int_ctrl;
    end
        
end

always@(posedge cpu_clk) begin
    if(!rstn) begin
        slv_cpu_cs <= 'b0;
        slv_cpu_op <= 'b0;
        slv_cpu_addr <= 'b0;
        slv_cpu_wr_data <= 'b0;
        slv_cpu_byte_en <= 'b0;
    end
    else if(cpu_rst) begin
        slv_cpu_cs <= 'b0;
        slv_cpu_op <= 'b0;
        slv_cpu_addr <= 'b0;
        slv_cpu_wr_data <= 'b0;
        slv_cpu_byte_en <= 'b0;
    end
    else if(slv_cpu_ack) begin
        slv_cpu_cs <= 'b0;
        slv_cpu_op <= 'b0;
        if( slv_cpu_addr[5:2]==4'b1100 )
            slv_cpu_addr[5:2] <= 'b0;
        else
            slv_cpu_addr[5:2] <= slv_cpu_addr[5:2] + 1'b1;
        slv_cpu_addr[1:0] <='d0;
        slv_cpu_addr[7:6] <='d0;
        slv_cpu_wr_data <= 'b0;
        slv_cpu_byte_en <= 'b0;
    end
    else if(cnt == 0) begin
        slv_cpu_cs <= 'b1;
        slv_cpu_op <= 'b1;
        slv_cpu_addr <= shs_addr;
        slv_cpu_wr_data <= {7'b0,shs_data,24'b0};
        slv_cpu_byte_en <= 4'h8;
    end
    else if(cnt == 1) begin
        slv_cpu_cs <= 'b1;
        slv_cpu_op <= 'b0;
        slv_cpu_addr <= shs_addr;
        slv_cpu_wr_data <= 32'b0;
        slv_cpu_byte_en <= 4'b0;
    end
    else if(cnt == 2) begin
        slv_cpu_cs <= 'b1;
        slv_cpu_op <= 'b1;
        slv_cpu_addr <= lsc_addr;
        slv_cpu_wr_data <= {9'b0,lsc_data,22'b0};
        slv_cpu_byte_en <= 4'h4;
    end
    else if(cnt == 3) begin
        slv_cpu_cs <= 'b1;
        slv_cpu_op <= 'b0;
        slv_cpu_addr <= lsc_addr;
        slv_cpu_wr_data <= 32'b0;
        slv_cpu_byte_en <= 4'b0;
    end
    else if(cnt == 4) begin
        slv_cpu_cs <= 'b1;
        slv_cpu_op <= 'b1;
        slv_cpu_addr <= cccr_cis_pointer_addr;
        slv_cpu_wr_data <= {8'b0,cccr_cis_pointer_data};
        slv_cpu_byte_en <= 4'h7;
    end
    else if(cnt == 5) begin
        slv_cpu_cs <= 'b1;
        slv_cpu_op <= 'b0;
        slv_cpu_addr <= cccr_cis_pointer_addr;
        slv_cpu_wr_data <= 32'b0;
        slv_cpu_byte_en <= 4'b0;
    end
    else if(cnt == 6) begin
        slv_cpu_cs <= 'b1;
        slv_cpu_op <= 'b1;
        slv_cpu_addr <= fbr1_cis_ptr_addr;
        slv_cpu_wr_data <= {8'b0,fbr1_cis_ptr_data};
        slv_cpu_byte_en <= 4'h7;
    end
    else if(cnt == 7) begin
        slv_cpu_cs <= 'b1;
        slv_cpu_op <= 'b0;
        slv_cpu_addr <= fbr1_cis_ptr_addr;
        slv_cpu_wr_data <= 32'b0;
        slv_cpu_byte_en <= 4'b0;
    end
    else if(cnt == 8) begin
        slv_cpu_cs <= 'b1;
        slv_cpu_op <= 'b1;
        slv_cpu_addr <= cccr_max_blk_size_addr;
        slv_cpu_wr_data <= {16'b0,cccr_max_blk_size_data};
        slv_cpu_byte_en <= 4'h3;
    end
    else if(cnt == 9) begin
        slv_cpu_cs <= 'b1;
        slv_cpu_op <= 'b0;
        slv_cpu_addr <= cccr_max_blk_size_addr;
        slv_cpu_wr_data <= 32'b0;
        slv_cpu_byte_en <= 4'b0;
    end
    else if(cnt == 10) begin
        slv_cpu_cs <= 'b1;
        slv_cpu_op <= 'b1;
        slv_cpu_addr <= fbr1_max_blk_size_addr;
        slv_cpu_wr_data <= {fbr1_max_blk_size_data,16'd0};
        slv_cpu_byte_en <= 4'hc;
    end
    else if(cnt == 11) begin
        slv_cpu_cs <= 'b1;
        slv_cpu_op <= 'b0;
        slv_cpu_addr <= fbr1_max_blk_size_addr;
        slv_cpu_wr_data <= 32'b0;
        slv_cpu_byte_en <= 4'b0;
    end
    else if(cnt == 12) begin
        slv_cpu_cs <= 'b1;
        slv_cpu_op <= 'b1;
        slv_cpu_addr <= io_ready_addr;
        slv_cpu_wr_data <= {6'b0,2'b00,23'b0,io_ready_data};
        slv_cpu_byte_en <= 4'h9;
    end
    else if(cnt == 13) begin
        slv_cpu_cs <= 'b1;
        slv_cpu_op <= 'b0;
        slv_cpu_addr <= io_ready_addr;
        slv_cpu_wr_data <= 32'b0;
        slv_cpu_byte_en <= 4'b0;
    end
    else begin
        slv_cpu_cs <= 'b1;
        slv_cpu_op <= 'b0;
        slv_cpu_addr <= slv_cpu_addr;
        slv_cpu_wr_data <= 32'b0;
        slv_cpu_byte_en <= 4'b0;
    end
end

always@(posedge cpu_clk or negedge rstn) begin
    if(!rstn)
        fun1_interrupt <= 0;
    else if(cpu_rst)
        fun1_interrupt <= 0;
    else if(turn_flag)
        fun1_interrupt <= 1;
    else if(int_clr_flag_d2)
        fun1_interrupt <= 0;
end

always@(posedge cpu_clk or negedge rstn) begin
    if(!rstn)
        fun1_ior <= 0;
    else if(cpu_rst)
        fun1_ior <= 0;
    else
        fun1_ior <= ior_ctrl;
end

endmodule