module user_sdio_cmd52_if(
//--------clock and reset---------
input           clk,  //sd clock
input           rstn, //system rstn
input           fun1_rstn,  

input               sdio_cmd52_cs,
input               sdio_cmd52_r_w,
input               sdio_cmd52_fn_num,
input               sdio_cmd52_raw,//1:The Read after Write flag
input [16:0]        sdio_cmd52_addr,
input [7:0]         sdio_cmd52_wr_data,
output [7:0]        sdio_cmd52_rd_data,
output              sdio_cmd52_ack
);

reg [3:0] ack_delay;
reg [3:0] ack_delay_cnt;


reg sdio_cmd52_cs_d1;
reg sdio_cmd52_r_w_d1;
reg sdio_cmd52_raw_d1;
reg [7:0] user_cmd52_read_reg;
reg [7:0] user_cmd52_write_return_reg;

always@(posedge clk or negedge rstn) begin
    if (!rstn) begin
        ack_delay <= 'b1;
    end
    else if (fun1_rstn) begin
        ack_delay <= 'b1;
    end
    else if (sdio_cmd52_ack) begin
        if (ack_delay == 4'b1111) begin
            ack_delay <= 'b1;
        end
        else begin
            ack_delay <= ack_delay + 1'b1;
        end
    end
end

always@(posedge clk or negedge rstn) begin
    if (!rstn) begin
        sdio_cmd52_cs_d1 <= 1'b0;
        sdio_cmd52_r_w_d1 <= 1'b0;
        sdio_cmd52_raw_d1 <= 1'b0;
    end
    else if (fun1_rstn) begin
        sdio_cmd52_cs_d1 <= 1'b0;
        sdio_cmd52_r_w_d1 <= 1'b0;
        sdio_cmd52_raw_d1 <= 1'b0;
    end
    else if (sdio_cmd52_ack) begin
        sdio_cmd52_cs_d1 <= 1'b0;
        sdio_cmd52_r_w_d1 <= 1'b0;
        sdio_cmd52_raw_d1 <= 1'b0;
    end
    else if (sdio_cmd52_cs) begin
        sdio_cmd52_cs_d1 <= sdio_cmd52_cs;
        sdio_cmd52_r_w_d1 <= sdio_cmd52_r_w;
        sdio_cmd52_raw_d1 <= sdio_cmd52_raw;
    end
end

always@(posedge clk or negedge rstn) begin
    if (!rstn) begin
        ack_delay_cnt <= 'b0;
    end
    else if (fun1_rstn) begin
        ack_delay_cnt <= 'b0;
    end
    else if (sdio_cmd52_ack) begin
        ack_delay_cnt <= 'b0;
    end
    else if (sdio_cmd52_cs || sdio_cmd52_cs_d1) begin
        ack_delay_cnt <= ack_delay_cnt + 1'b1;
    end
end

assign sdio_cmd52_rd_data = (sdio_cmd52_ack && ((sdio_cmd52_cs && !sdio_cmd52_r_w) || (sdio_cmd52_cs_d1 && !sdio_cmd52_r_w_d1))) ? user_cmd52_read_reg : (sdio_cmd52_ack && ((sdio_cmd52_cs && sdio_cmd52_r_w && sdio_cmd52_raw) || (sdio_cmd52_cs_d1 && sdio_cmd52_r_w_d1 && sdio_cmd52_raw_d1))) ? user_cmd52_write_return_reg : 'b0;
assign sdio_cmd52_ack = (ack_delay == 'b0) ? sdio_cmd52_cs : ((sdio_cmd52_cs || sdio_cmd52_cs_d1) && (ack_delay_cnt == ack_delay)) ? 1'b1 : 1'b0;

//----calculate read value
always@(posedge clk or negedge rstn) begin
    if (!rstn) begin
        user_cmd52_read_reg <= 'b0;
    end
    else if (fun1_rstn) begin
        user_cmd52_read_reg <= 'b0;
    end
    else if (sdio_cmd52_cs && !sdio_cmd52_r_w) begin
        user_cmd52_read_reg <= 8'h5c;
    end
end

//----calculate write value
always@(posedge clk or negedge rstn) begin
    if (!rstn) begin
        user_cmd52_write_return_reg <= 'b0;
    end
    else if (fun1_rstn) begin
        user_cmd52_write_return_reg <= 'b0;
    end
    else if (sdio_cmd52_cs && sdio_cmd52_r_w) begin
        user_cmd52_write_return_reg <= 8'h9b;
    end
end

endmodule


