module user_sdio_cmd53_if(
//--------clock and reset---------
input           clk,  //sd clock
input           rstn, //system rstn
input           fun1_rst,  
//cmd53
input           sdio_cmd53_wr_en,
input           sdio_cmd53_rd_en,
input           sdio_cmd53_fn_num,
input [16:0]    sdio_cmd53_addr,
input [11:0]    sdio_cmd53_len,
input           sdio_cmd53_op_code,
    
input           sdio_cmd53_wr_valid,
input [7:0]     sdio_cmd53_wr_data,
input           sdio_cmd53_wr_end,
input           sdio_cmd53_wr_ok,
input           sdio_cmd53_wr_abort,

output reg      sdio_cmd53_rd_valid,
output [7:0]    sdio_cmd53_rd_data,
input           sdio_cmd53_rd_ready,
input           sdio_cmd53_rd_end,
input           sdio_cmd53_rd_abort,

output reg      sdio_buffer_full

);

reg [2:0] c_state,n_state;
localparam s_idle       = 3'd0;
localparam s_wr         = 3'd1;
localparam s_wr_wt_chk  = 3'd2;
localparam s_rd         = 3'd3;

reg [11:0] cnt;

reg           sdio_cmd53_wr_en_d1;
reg           sdio_cmd53_rd_en_d1;
reg           sdio_cmd53_fn_num_d1;
reg [16:0]    sdio_cmd53_addr_d1;
reg [11:0]    sdio_cmd53_len_d1;
reg           sdio_cmd53_op_code_d1;

reg           sdio_cmd53_wr_end_d1;
reg           sdio_cmd53_wr_ok_d1;

reg [11:0]    blk_cnt = 0;
reg [11:0]    blk_num = 0;
reg           infinite = 0;
reg           blk_flag = 0;

reg [7:0] buf_full_cnt;
localparam buf_full_par = 8'h9;
reg [3:0] idle_to_rd_delay;
reg [3:0] idle_to_rd_delay_cnt;

reg [7:0] user_cmd53_rd_data_reg;

always@(posedge clk or negedge rstn) begin
    if (!rstn) begin
        idle_to_rd_delay <= 4'd0;
    end
    else if (fun1_rst) begin
        idle_to_rd_delay <= 4'd0;
    end
    else if (c_state == s_rd && n_state == s_idle) begin
        idle_to_rd_delay <= idle_to_rd_delay + 1'b1;
    end
end

always@(posedge clk or negedge rstn) begin
    if (!rstn) begin
        idle_to_rd_delay_cnt <= 4'd0;
    end
    else if (fun1_rst) begin
        idle_to_rd_delay_cnt <= 4'd0;
    end
    else if (c_state == s_idle) begin
        idle_to_rd_delay_cnt <= 4'd0;
    end
    else if (c_state == s_rd && idle_to_rd_delay_cnt != idle_to_rd_delay) begin
        idle_to_rd_delay_cnt <= idle_to_rd_delay_cnt + 1'b1;
    end
end


always@(posedge clk or negedge rstn) begin
    if (!rstn) begin
        sdio_cmd53_rd_valid <= 1'b0;
    end
    else if (c_state == s_rd && n_state == s_idle) begin
        sdio_cmd53_rd_valid <= 1'b0;
    end
    else if (c_state == s_rd && idle_to_rd_delay_cnt == idle_to_rd_delay) begin
        sdio_cmd53_rd_valid <= 1'b1;
    end
end
assign sdio_cmd53_rd_data = (c_state == s_rd) ? user_cmd53_rd_data_reg : 'b0;


always@(posedge clk or negedge rstn) begin
    if (!rstn) begin
        c_state <= s_idle;
    end
    else if (fun1_rst) begin
        c_state <= s_idle;
    end
    else begin
        c_state <= n_state;
    end
end

always@(*) begin
    n_state = c_state;
    case(c_state) 
    s_idle:
        if (sdio_cmd53_wr_en) n_state = s_wr;
        else if (sdio_cmd53_rd_en) n_state = s_rd;
    s_wr:
        if (sdio_cmd53_wr_abort) n_state = s_idle;
        else if (cnt+1 == sdio_cmd53_len_d1 && sdio_cmd53_wr_valid) n_state = s_wr_wt_chk;
    s_wr_wt_chk:
        if (sdio_cmd53_wr_end|| sdio_cmd53_wr_abort) n_state = s_idle;
    s_rd:
        if ((cnt+1 == sdio_cmd53_len_d1 && sdio_cmd53_rd_ready) || sdio_cmd53_rd_abort)  n_state = s_idle;
    endcase
end


always@(posedge clk or negedge rstn) begin
    if (!rstn) begin
        sdio_buffer_full <= 'd0;
    end
    else if (fun1_rst) begin
        sdio_buffer_full <= 'd0;
    end
    else if (c_state == s_wr_wt_chk && n_state == s_idle) begin
        sdio_buffer_full <= 1'b1;
    end
    else if (buf_full_cnt == buf_full_par) begin
        sdio_buffer_full <= 'd0;
    end
end
always@(posedge clk or negedge rstn) begin
    if (!rstn) begin
        buf_full_cnt <= 'd0;
    end
    else if (fun1_rst) begin
        buf_full_cnt <= 'd0;
    end
    else if (buf_full_cnt == buf_full_par) begin
        buf_full_cnt <= 'd0;
    end
    else if (sdio_buffer_full) begin
        buf_full_cnt <= buf_full_cnt + 1'b1;
    end
end

always@(posedge clk or negedge rstn) begin
    if (!rstn) begin
        cnt <= 'd0;
    end
    else if (fun1_rst) begin
        cnt <= 'd0;
    end
    else if (c_state == s_idle) begin
        cnt <= 'd0;
    end
    else if ((c_state == s_wr && sdio_cmd53_wr_valid) || (c_state == s_rd && sdio_cmd53_rd_ready)) begin
        cnt <= cnt + 1'b1;
    end
end
    
always@(posedge clk or negedge rstn) begin
    if (!rstn) begin
        sdio_cmd53_wr_en_d1 <= 'b0;
        sdio_cmd53_rd_en_d1 <= 'b0;
        sdio_cmd53_fn_num_d1 <= 'b0;
        sdio_cmd53_addr_d1 <= 'b0;
        sdio_cmd53_len_d1 <= 'b0;
        sdio_cmd53_op_code_d1 <= 'b0;
        sdio_cmd53_wr_end_d1 <= 'b0;
        sdio_cmd53_wr_ok_d1 <= 'b0;     
    end
    else if (fun1_rst) begin
        sdio_cmd53_wr_en_d1 <= 'b0;
        sdio_cmd53_rd_en_d1 <= 'b0;
        sdio_cmd53_fn_num_d1 <= 'b0;
        sdio_cmd53_addr_d1 <= 'b0;
        sdio_cmd53_len_d1 <= 'b0;
        sdio_cmd53_op_code_d1 <= 'b0;
        sdio_cmd53_wr_end_d1 <= 'b0;
        sdio_cmd53_wr_ok_d1 <= 'b0;    
    end
    else if (c_state == s_idle && (sdio_cmd53_wr_en || sdio_cmd53_rd_en)) begin
        sdio_cmd53_wr_en_d1     <= sdio_cmd53_wr_en;
        sdio_cmd53_rd_en_d1     <= sdio_cmd53_rd_en;
        sdio_cmd53_fn_num_d1    <= sdio_cmd53_fn_num;
        sdio_cmd53_addr_d1      <= sdio_cmd53_addr;
        sdio_cmd53_len_d1       <= sdio_cmd53_len;
        sdio_cmd53_op_code_d1   <= sdio_cmd53_op_code;
        sdio_cmd53_wr_end_d1    <= sdio_cmd53_wr_end;
        sdio_cmd53_wr_ok_d1     <= sdio_cmd53_wr_ok; 
    end
end

//---------rd data gen and check--------
always@(posedge clk or negedge rstn) begin
    if (!rstn) begin
        user_cmd53_rd_data_reg <= 'b0;
    end
    else if (fun1_rst) begin
        user_cmd53_rd_data_reg <= 'b0;
    end
    else if ((c_state == s_idle && n_state == s_rd) || (sdio_cmd53_rd_valid && sdio_cmd53_rd_ready)) begin
        user_cmd53_rd_data_reg <= 8'h6d;
    end
end

endmodule